import k8scmd

__author__ = "shigebeyond"
__version__ = "1.0.0"
__description__ = "k8scmd: easy way to execute k8s command"

__all__ = [
    "__author__",
    "__version__",
    "__description__",
    "k8scmd",
]